/*
 * Copyright 2005-2006 Myriad Genetics, Inc.
 * Copyright 2008, 2010 Dustin Cartwright
 *
 * All rights reserved.
 */

package com.myriad.tmap.grouping;

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.NumberFormat;
import javax.swing.*;
import com.myriad.tmap.compute.Pedigree;
import com.myriad.tmap.Quitter;

/** GroupingComponent wraps a GroupingTreeComponent with a scroll pane and a few
    buttons and controls to interact with the GroupingTreeComponent.
 */
public class GroupingComponent extends JComponent {
    /** Creates a GroupingComponent component.
        @param aPed the pedigree information to display
        @param minLod the initial minimum lod score
        @param maxLod the initial maximum lod score
        @param maxDist the initial maximum distance between linked markers
     */
    public GroupingComponent(Pedigree aPed, double minLod, double maxLod,
                             double maxDist)
    {
        JPanel              northPanel, southPanel;
        JFormattedTextField distField, minField, maxField;
        JButton             button;
        
        ped = aPed;
        chooser = null;
        
        setLayout(new BorderLayout());
        
        groupingComp = new GroupingTreeComponent(ped, minLod, maxLod, maxDist);
        scrollPane = new JScrollPane(groupingComp);
        add(scrollPane, BorderLayout.CENTER);
        
        northPanel = new JPanel();
        
        add(northPanel, BorderLayout.NORTH);
        
        northPanel.add(new JLabel("Max dist:"));
        
        distField = new JFormattedTextField(NumberFormat.getInstance());
        distField.setValue(new Double(maxDist));
        distField.setColumns(5);
        distField.addPropertyChangeListener(new distListener());
        northPanel.add(distField);
        
        northPanel.add(new JLabel("Min lod:"));
        
        minField = new JFormattedTextField(NumberFormat.getInstance());
        minField.setValue(new Double(minLod));
        minField.setColumns(5);
        minField.addPropertyChangeListener(new minListener());
        northPanel.add(minField);
        
        northPanel.add(new JLabel("Max lod:"));
        
        maxField = new JFormattedTextField(NumberFormat.getInstance());
        maxField.setValue(new Double(maxLod));
        maxField.setColumns(5);
        maxField.addPropertyChangeListener(new maxListener());
        northPanel.add(maxField);
        
        southPanel = new JPanel();
        add(southPanel, BorderLayout.SOUTH);
        
        button = new JButton("Load");
        button.addActionListener(new LoadListener());
        southPanel.add(button);
        
        button = new JButton("Save");
        button.addActionListener(new SaveListener());
        southPanel.add(button);
        
        button = new JButton("Build");
        button.addActionListener(new BuildListener(false));
        southPanel.add(button);
        
        button = new JButton("Build Split");
        button.addActionListener(new BuildListener(true));
        southPanel.add(button);
    }

    public void putInNewFrame()
    {
        JFrame  frame = new JFrame("Grouping markers in " + ped.getFilename());
        
        frame.getContentPane().add(this);
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        frame.addWindowListener(new Quitter());
        frame.setSize(600, 600);
        frame.setVisible(true);
    }
    
    private class distListener implements PropertyChangeListener
    {
        public void propertyChange(PropertyChangeEvent evt)
        {
            if (evt.getPropertyName().equals("value")) {
                Number  n = (Number) evt.getNewValue();
                
                groupingComp.setMaxDist(n.doubleValue());
            }
        }
    }
    
    private class minListener implements PropertyChangeListener
    {
        public void propertyChange(PropertyChangeEvent evt)
        {
            if (evt.getPropertyName().equals("value")) {
                Number  n = (Number) evt.getNewValue();
                
                groupingComp.setMinLod(n.doubleValue());
            }
        }
    }
    
    private class maxListener implements PropertyChangeListener
    {
        public void propertyChange(PropertyChangeEvent evt)
        {
            if (evt.getPropertyName().equals("value")) {
                Number  n = (Number) evt.getNewValue();
                
                groupingComp.setMaxLod(n.doubleValue());
            }
        }
    }
    
    private class LoadListener implements ActionListener
    {
        public void actionPerformed(ActionEvent event)
        {
            if (chooser == null) {
                chooser = new JFileChooser();
                chooser.setDialogTitle("Choose linkage group file");
            }
            
            if (chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
                String      path = chooser.getSelectedFile().getAbsolutePath();
                String[]    markNames = ped.getOrder(path);
                int         pos;
                
                if (markNames == null)
                    return;
                
                groupingComp.clearSelectedMarkers();
                pos = groupingComp.selectMarkersByName(markNames);
                scrollPane.getViewport().setViewPosition(new Point(0, pos));
                
                ChooserFileIncrementer.increment(chooser, true);
            }
        }
    }
    
    private class SaveListener implements ActionListener
    {
        public void actionPerformed(ActionEvent event)
        {
            groupingComp.saveSelectedMarkers();
        }
    }
    
    private class BuildListener implements ActionListener
    {
        BuildListener(boolean aSplit)
        {
            split = aSplit;
        }
        
        public void actionPerformed(ActionEvent event)
        {
            groupingComp.makeMapFrame(split);
        }
        
        boolean split;
    }
    
    private GroupingTreeComponent   groupingComp;
    private Pedigree                ped;
    private JFileChooser            chooser;
    private JScrollPane             scrollPane;
}
