/*
 * Copyright 2007, 2010 Dustin Cartwright
 *
 * All rights reserved.
 */

package com.myriad.tmap.improve;

import java.awt.EventQueue;
import java.awt.Point;
import com.myriad.tmap.compute.Pedigree;
import com.myriad.tmap.WorkingDialog;
import com.myriad.tmap.improve.MapComponent;

/** This class represents a separate thread which makes a new map, either
    through the "improve" function or "build," and then displays it in a new
    MapComponent frame.
 */
public class MakeNewMapThread implements Runnable {
    public MakeNewMapThread(Pedigree aPed, String[] aMarkNames, boolean aSplit,
                            Point aParentLoc, boolean aImprove, String aName)
    {
        ped = aPed;
        markNames = aMarkNames;
        split = aSplit;
        parentLoc = aParentLoc;
        improve = aImprove;
        working = new WorkingDialog((improve ? "Improving" : "Building")
                + " linkage group " + aName + " from " + ped.getFilename());
        name = aName;

        (new Thread(this)).start();
    }
    
    public void run()
    {
        if (improve)
            ped.improve(markNames, split);
        else
            ped.build(markNames, split);

        EventQueue.invokeLater(new Runnable() {
            public void run() {
                MapComponent    mapComp;

                /* :TODO: should pass the map structure along here. */
                mapComp = new MapComponent(ped, markNames, split, name);
                mapComp.putInNewFrame(parentLoc);
                working.dispose();
            }
        });
    }
    
    private Pedigree        ped;
    private String[]        markNames;
    private boolean         split;
    private Point           parentLoc;
    private boolean         improve;
    private WorkingDialog   working;
    private String          name;
}
