/*
 * Copyright 2005-2006 Myriad Genetics, Inc.
 * Copyright 2010 Dustin Cartwright
 *
 * All rights reserved.
 */

#include "genmap.h"
#include "input.h"
#include "output.h"
#include "outputPhasing.h"
#include "ped.h"
#include "Phasing.h"
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#define USAGE_STRING "usage: %s [-j <JoinMap file>] <input file> <output file> [<lod thresh>]\n"

int main(int argc, char **argv)
{
    double          lodThresh = 0.0;
    const char      *inFile;
    const char      *outFile;
    const char      *joinMapFile = NULL;
    swap_t          *swaps = NULL;
    pedigree_t      ped;
    bool            varyThresh;
    int             c;

    while ((c = getopt(argc, argv, "j:")) != EOF) {
        switch (c) {
            case 'j':
                joinMapFile = optarg;
                break;
        }
    }

    if (argc - optind == 2) {
        varyThresh = true;
    } else if (argc - optind == 3) {
        char    *endp;
        
        varyThresh = false;
        
        lodThresh = strtod(argv[optind + 2], &endp);
        if (*endp != '\0') {
            fprintf(stderr, "%s: The lod threshold %s is not a valid number\n",
                    *argv, argv[optind + 2]);
            fprintf(stderr, USAGE_STRING, *argv);
            return EXIT_FAILURE;
        }
    } else {
        fprintf(stderr, USAGE_STRING, argv[0]);
        return EXIT_FAILURE;
    }
    
    inFile = argv[optind];
    outFile = argv[optind + 1];
    
    if (!readFile(&ped, inFile))
        return EXIT_FAILURE;

    if (joinMapFile)
        swaps = (swap_t *)malloc(ped.numMarks * sizeof(swap_t));
    
    if (varyThresh) {
        lodThresh = phasingVary(&ped, swaps);
        printf("Used a LOD threshold of %.4f\n", lodThresh);
    } else {
        if (!phasing(&ped, lodThresh, swaps)) {
            printf("Could not determine the phases. ");
            printf("Try using a higher LOD threshold.\n");
            return EXIT_SUCCESS;
        }
    }
    
    writePed(&ped, outFile);
    if (joinMapFile)
        writeJoinMapFile(joinMapFile, &ped, swaps);
    return EXIT_SUCCESS;
}
